@@ -169,44 +169,6 @@ public interface TemplateService extends Service {
      */
     RemoveRequest remove();
     
-    /**
-     * Seal the template.
-     * 
-     * Sealing erases all host-specific configuration from the filesystem:
-     * SSH keys, UDEV rules, MAC addresses, system ID, hostname etc.,
-     * thus making easy to use the template to create multiple virtual
-     * machines without manual intervention.
-     * 
-     * Currently sealing is supported only for Linux OS.
-     */
-    public interface SealRequest extends Request<SealRequest, SealResponse> {
-    }
-    
-    /**
-     * Seal the template.
-     * 
-     * Sealing erases all host-specific configuration from the filesystem:
-     * SSH keys, UDEV rules, MAC addresses, system ID, hostname etc.,
-     * thus making easy to use the template to create multiple virtual
-     * machines without manual intervention.
-     * 
-     * Currently sealing is supported only for Linux OS.
-     */
-    public interface SealResponse extends Response {
-    }
-    
-    /**
-     * Seal the template.
-     * 
-     * Sealing erases all host-specific configuration from the filesystem:
-     * SSH keys, UDEV rules, MAC addresses, system ID, hostname etc.,
-     * thus making easy to use the template to create multiple virtual
-     * machines without manual intervention.
-     * 
-     * Currently sealing is supported only for Linux OS.
-     */
-    SealRequest seal();
-    
     /**
      * Updates the template.
      * 
