@@ -91,6 +91,20 @@ public interface TemplatesService extends Service {
          * ----
          */
         AddRequest clonePermissions(Boolean clonePermissions);
+        /**
+         * Seal the template.
+         * 
+         * If this optional parameter is provided and its value is `true`,
+         * then the template is sealed after creation.
+         * 
+         * Sealing erases all host-specific configuration from the filesystem:
+         * SSH keys, UDEV rules, MAC addresses, system ID, hostname etc.,
+         * thus making easy to use the template to create multiple virtual
+         * machines without manual intervention.
+         * 
+         * Currently sealing is supported only for Linux OS.
+         */
+        AddRequest seal(Boolean seal);
         AddRequest template(Template template);
         /**
          * The information about the template or template version.
