@@ -23,6 +23,7 @@ import java.net.URISyntaxException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
+import java.util.Map;
 import org.apache.http.HttpResponse;
 import org.apache.http.client.methods.HttpDelete;
 import org.apache.http.client.methods.HttpGet;
@@ -33,6 +34,7 @@ import org.apache.http.entity.ByteArrayEntity;
 import org.apache.http.util.EntityUtils;
 import org.ovirt.api.metamodel.runtime.xml.XmlReader;
 import org.ovirt.api.metamodel.runtime.xml.XmlWriter;
+import org.ovirt.engine.sdk4.BaseRequest;
 import org.ovirt.engine.sdk4.Error;
 import org.ovirt.engine.sdk4.Service;
 import org.ovirt.engine.sdk4.builders.ActionBuilder;
@@ -79,7 +81,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         super(connection, path);
     }
     
-    private class ActivateRequestImpl implements ActivateRequest {
+    private class ActivateRequestImpl extends BaseRequest<ActivateRequest, ActivateResponse> implements ActivateRequest {
         private Boolean async;
         public ActivateRequest async(Boolean async) {
             this.async = async;
@@ -102,6 +104,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -117,7 +125,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new ActivateRequestImpl();
     }
     
-    private class ApproveRequestImpl implements ApproveRequest {
+    private class ApproveRequestImpl extends BaseRequest<ApproveRequest, ApproveResponse> implements ApproveRequest {
         private Boolean async;
         public ApproveRequest async(Boolean async) {
             this.async = async;
@@ -152,6 +160,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -167,7 +181,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new ApproveRequestImpl();
     }
     
-    private class CommitNetConfigRequestImpl implements CommitNetConfigRequest {
+    private class CommitNetConfigRequestImpl extends BaseRequest<CommitNetConfigRequest, CommitNetConfigResponse> implements CommitNetConfigRequest {
         private Boolean async;
         public CommitNetConfigRequest async(Boolean async) {
             this.async = async;
@@ -190,6 +204,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -205,7 +225,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new CommitNetConfigRequestImpl();
     }
     
-    private class DeactivateRequestImpl implements DeactivateRequest {
+    private class DeactivateRequestImpl extends BaseRequest<DeactivateRequest, DeactivateResponse> implements DeactivateRequest {
         private Boolean async;
         public DeactivateRequest async(Boolean async) {
             this.async = async;
@@ -242,6 +262,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -257,7 +283,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new DeactivateRequestImpl();
     }
     
-    private class EnrollCertificateRequestImpl implements EnrollCertificateRequest {
+    private class EnrollCertificateRequestImpl extends BaseRequest<EnrollCertificateRequest, EnrollCertificateResponse> implements EnrollCertificateRequest {
         private Boolean async;
         public EnrollCertificateRequest async(Boolean async) {
             this.async = async;
@@ -280,6 +306,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -295,7 +327,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new EnrollCertificateRequestImpl();
     }
     
-    private class FenceRequestImpl implements FenceRequest {
+    private class FenceRequestImpl extends BaseRequest<FenceRequest, FenceResponse> implements FenceRequest {
         private Boolean async;
         public FenceRequest async(Boolean async) {
             this.async = async;
@@ -325,6 +357,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -350,7 +388,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new FenceRequestImpl();
     }
     
-    private class ForceSelectSpmRequestImpl implements ForceSelectSpmRequest {
+    private class ForceSelectSpmRequestImpl extends BaseRequest<ForceSelectSpmRequest, ForceSelectSpmResponse> implements ForceSelectSpmRequest {
         private Boolean async;
         public ForceSelectSpmRequest async(Boolean async) {
             this.async = async;
@@ -373,6 +411,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -388,7 +432,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new ForceSelectSpmRequestImpl();
     }
     
-    private class GetRequestImpl implements GetRequest {
+    private class GetRequestImpl extends BaseRequest<GetRequest, GetResponse> implements GetRequest {
         private Boolean filter;
         public GetRequest filter(Boolean filter) {
             this.filter = filter;
@@ -402,12 +446,23 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 if (filter != null) {
                     uriBuilder.addParameter("filter", XmlWriter.renderBoolean(filter));
                 }
+                if (query != null) {
+                    for (Map.Entry<String, String> queryParam : query.entrySet()) {
+                        uriBuilder.addParameter(queryParam.getKey(), queryParam.getValue());
+                    }
+                }
                 request = new HttpGet(uriBuilder.build());
             }
             catch (URISyntaxException ex) {
                 throw new Error("Failed to build URL", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             if (
               response.getStatusLine().getStatusCode() == 200
@@ -449,7 +504,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new GetRequestImpl();
     }
     
-    private class InstallRequestImpl implements InstallRequest {
+    private class InstallRequestImpl extends BaseRequest<InstallRequest, InstallResponse> implements InstallRequest {
         private Boolean async;
         public InstallRequest async(Boolean async) {
             this.async = async;
@@ -524,6 +579,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -539,7 +600,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new InstallRequestImpl();
     }
     
-    private class IscsiDiscoverRequestImpl implements IscsiDiscoverRequest {
+    private class IscsiDiscoverRequestImpl extends BaseRequest<IscsiDiscoverRequest, IscsiDiscoverResponse> implements IscsiDiscoverRequest {
         private Boolean async;
         public IscsiDiscoverRequest async(Boolean async) {
             this.async = async;
@@ -574,6 +635,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -599,7 +666,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new IscsiDiscoverRequestImpl();
     }
     
-    private class IscsiLoginRequestImpl implements IscsiLoginRequest {
+    private class IscsiLoginRequestImpl extends BaseRequest<IscsiLoginRequest, IscsiLoginResponse> implements IscsiLoginRequest {
         private Boolean async;
         public IscsiLoginRequest async(Boolean async) {
             this.async = async;
@@ -634,6 +701,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -649,7 +722,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new IscsiLoginRequestImpl();
     }
     
-    private class RefreshRequestImpl implements RefreshRequest {
+    private class RefreshRequestImpl extends BaseRequest<RefreshRequest, RefreshResponse> implements RefreshRequest {
         private Boolean async;
         public RefreshRequest async(Boolean async) {
             this.async = async;
@@ -672,6 +745,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -687,7 +766,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new RefreshRequestImpl();
     }
     
-    private class RemoveRequestImpl implements RemoveRequest {
+    private class RemoveRequestImpl extends BaseRequest<RemoveRequest, RemoveResponse> implements RemoveRequest {
         private Boolean async;
         public RemoveRequest async(Boolean async) {
             this.async = async;
@@ -701,12 +780,23 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 if (async != null) {
                     uriBuilder.addParameter("async", XmlWriter.renderBoolean(async));
                 }
+                if (query != null) {
+                    for (Map.Entry<String, String> queryParam : query.entrySet()) {
+                        uriBuilder.addParameter(queryParam.getKey(), queryParam.getValue());
+                    }
+                }
                 request = new HttpDelete(uriBuilder.build());
             }
             catch (URISyntaxException ex) {
                 throw new Error("Failed to build URL", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             if (
               response.getStatusLine().getStatusCode() == 200
@@ -729,7 +819,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new RemoveRequestImpl();
     }
     
-    private class SetupNetworksRequestImpl implements SetupNetworksRequest {
+    private class SetupNetworksRequestImpl extends BaseRequest<SetupNetworksRequest, SetupNetworksResponse> implements SetupNetworksRequest {
         private Boolean async;
         public SetupNetworksRequest async(Boolean async) {
             this.async = async;
@@ -916,6 +1006,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -931,7 +1027,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new SetupNetworksRequestImpl();
     }
     
-    private class UnregisteredStorageDomainsDiscoverRequestImpl implements UnregisteredStorageDomainsDiscoverRequest {
+    private class UnregisteredStorageDomainsDiscoverRequestImpl extends BaseRequest<UnregisteredStorageDomainsDiscoverRequest, UnregisteredStorageDomainsDiscoverResponse> implements UnregisteredStorageDomainsDiscoverRequest {
         private Boolean async;
         public UnregisteredStorageDomainsDiscoverRequest async(Boolean async) {
             this.async = async;
@@ -966,6 +1062,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -991,7 +1093,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new UnregisteredStorageDomainsDiscoverRequestImpl();
     }
     
-    private class UpdateRequestImpl implements UpdateRequest {
+    private class UpdateRequestImpl extends BaseRequest<UpdateRequest, UpdateResponse> implements UpdateRequest {
         private Boolean async;
         public UpdateRequest async(Boolean async) {
             this.async = async;
@@ -1016,6 +1118,11 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 if (async != null) {
                     uriBuilder.addParameter("async", XmlWriter.renderBoolean(async));
                 }
+                if (query != null) {
+                    for (Map.Entry<String, String> queryParam : query.entrySet()) {
+                        uriBuilder.addParameter(queryParam.getKey(), queryParam.getValue());
+                    }
+                }
                 request = new HttpPut(uriBuilder.build());
             }
             catch (URISyntaxException ex) {
@@ -1033,6 +1140,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to parse response", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             if (
               response.getStatusLine().getStatusCode() == 200
@@ -1074,7 +1187,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new UpdateRequestImpl();
     }
     
-    private class UpgradeRequestImpl implements UpgradeRequest {
+    private class UpgradeRequestImpl extends BaseRequest<UpgradeRequest, UpgradeResponse> implements UpgradeRequest {
         private Boolean async;
         public UpgradeRequest async(Boolean async) {
             this.async = async;
@@ -1097,6 +1210,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -1112,7 +1231,7 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
         return new UpgradeRequestImpl();
     }
     
-    private class UpgradeCheckRequestImpl implements UpgradeCheckRequest {
+    private class UpgradeCheckRequestImpl extends BaseRequest<UpgradeCheckRequest, UpgradeCheckResponse> implements UpgradeCheckRequest {
         public UpgradeCheckResponse send() {
             HttpPost request = new HttpPost(getConnection().getUrl() + getPath() + "/upgradecheck");
             try (
@@ -1128,6 +1247,12 @@ public class HostServiceImpl extends ServiceImpl implements HostService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
