@@ -18,6 +18,7 @@ package org.ovirt.engine.sdk4.internal.services;
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.net.URISyntaxException;
+import java.util.Map;
 import org.apache.http.HttpResponse;
 import org.apache.http.client.methods.HttpGet;
 import org.apache.http.client.methods.HttpPost;
@@ -26,6 +27,7 @@ import org.apache.http.entity.ByteArrayEntity;
 import org.apache.http.util.EntityUtils;
 import org.ovirt.api.metamodel.runtime.xml.XmlReader;
 import org.ovirt.api.metamodel.runtime.xml.XmlWriter;
+import org.ovirt.engine.sdk4.BaseRequest;
 import org.ovirt.engine.sdk4.Error;
 import org.ovirt.engine.sdk4.Service;
 import org.ovirt.engine.sdk4.builders.ActionBuilder;
@@ -80,17 +82,28 @@ public class SystemServiceImpl extends ServiceImpl implements SystemService {
         super(connection, path);
     }
     
-    private class GetRequestImpl implements GetRequest {
+    private class GetRequestImpl extends BaseRequest<GetRequest, GetResponse> implements GetRequest {
         public GetResponse send() {
             HttpGet request = null;
             try {
                 URIBuilder uriBuilder = new URIBuilder(getConnection().getUrl() + getPath());
+                if (query != null) {
+                    for (Map.Entry<String, String> queryParam : query.entrySet()) {
+                        uriBuilder.addParameter(queryParam.getKey(), queryParam.getValue());
+                    }
+                }
                 request = new HttpGet(uriBuilder.build());
             }
             catch (URISyntaxException ex) {
                 throw new Error("Failed to build URL", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             if (
               response.getStatusLine().getStatusCode() == 200
@@ -132,7 +145,7 @@ public class SystemServiceImpl extends ServiceImpl implements SystemService {
         return new GetRequestImpl();
     }
     
-    private class ReloadConfigurationsRequestImpl implements ReloadConfigurationsRequest {
+    private class ReloadConfigurationsRequestImpl extends BaseRequest<ReloadConfigurationsRequest, ReloadConfigurationsResponse> implements ReloadConfigurationsRequest {
         private Boolean async;
         public ReloadConfigurationsRequest async(Boolean async) {
             this.async = async;
@@ -155,6 +168,12 @@ public class SystemServiceImpl extends ServiceImpl implements SystemService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
