@@ -21,6 +21,7 @@ import java.net.URISyntaxException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
+import java.util.Map;
 import org.apache.http.HttpResponse;
 import org.apache.http.client.methods.HttpDelete;
 import org.apache.http.client.methods.HttpGet;
@@ -31,6 +32,7 @@ import org.apache.http.entity.ByteArrayEntity;
 import org.apache.http.util.EntityUtils;
 import org.ovirt.api.metamodel.runtime.xml.XmlReader;
 import org.ovirt.api.metamodel.runtime.xml.XmlWriter;
+import org.ovirt.engine.sdk4.BaseRequest;
 import org.ovirt.engine.sdk4.Error;
 import org.ovirt.engine.sdk4.Service;
 import org.ovirt.engine.sdk4.builders.ActionBuilder;
@@ -77,7 +79,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         super(connection, path);
     }
     
-    private class CancelMigrationRequestImpl implements CancelMigrationRequest {
+    private class CancelMigrationRequestImpl extends BaseRequest<CancelMigrationRequest, CancelMigrationResponse> implements CancelMigrationRequest {
         private Boolean async;
         public CancelMigrationRequest async(Boolean async) {
             this.async = async;
@@ -100,6 +102,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -115,7 +123,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new CancelMigrationRequestImpl();
     }
     
-    private class CloneRequestImpl implements CloneRequest {
+    private class CloneRequestImpl extends BaseRequest<CloneRequest, CloneResponse> implements CloneRequest {
         private Boolean async;
         public CloneRequest async(Boolean async) {
             this.async = async;
@@ -150,6 +158,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -165,7 +179,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new CloneRequestImpl();
     }
     
-    private class CommitSnapshotRequestImpl implements CommitSnapshotRequest {
+    private class CommitSnapshotRequestImpl extends BaseRequest<CommitSnapshotRequest, CommitSnapshotResponse> implements CommitSnapshotRequest {
         private Boolean async;
         public CommitSnapshotRequest async(Boolean async) {
             this.async = async;
@@ -188,6 +202,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -203,7 +223,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new CommitSnapshotRequestImpl();
     }
     
-    private class DetachRequestImpl implements DetachRequest {
+    private class DetachRequestImpl extends BaseRequest<DetachRequest, DetachResponse> implements DetachRequest {
         private Boolean async;
         public DetachRequest async(Boolean async) {
             this.async = async;
@@ -226,6 +246,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -241,7 +267,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new DetachRequestImpl();
     }
     
-    private class ExportRequestImpl implements ExportRequest {
+    private class ExportRequestImpl extends BaseRequest<ExportRequest, ExportResponse> implements ExportRequest {
         private Boolean async;
         public ExportRequest async(Boolean async) {
             this.async = async;
@@ -290,6 +316,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -305,7 +337,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new ExportRequestImpl();
     }
     
-    private class FreezeFilesystemsRequestImpl implements FreezeFilesystemsRequest {
+    private class FreezeFilesystemsRequestImpl extends BaseRequest<FreezeFilesystemsRequest, FreezeFilesystemsResponse> implements FreezeFilesystemsRequest {
         private Boolean async;
         public FreezeFilesystemsRequest async(Boolean async) {
             this.async = async;
@@ -328,6 +360,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -343,7 +381,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new FreezeFilesystemsRequestImpl();
     }
     
-    private class GetRequestImpl implements GetRequest {
+    private class GetRequestImpl extends BaseRequest<GetRequest, GetResponse> implements GetRequest {
         private Boolean allContent;
         public GetRequest allContent(Boolean allContent) {
             this.allContent = allContent;
@@ -375,12 +413,23 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 if (nextRun != null) {
                     uriBuilder.addParameter("next_run", XmlWriter.renderBoolean(nextRun));
                 }
+                if (query != null) {
+                    for (Map.Entry<String, String> queryParam : query.entrySet()) {
+                        uriBuilder.addParameter(queryParam.getKey(), queryParam.getValue());
+                    }
+                }
                 request = new HttpGet(uriBuilder.build());
             }
             catch (URISyntaxException ex) {
                 throw new Error("Failed to build URL", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             if (
               response.getStatusLine().getStatusCode() == 200
@@ -422,7 +471,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new GetRequestImpl();
     }
     
-    private class LogonRequestImpl implements LogonRequest {
+    private class LogonRequestImpl extends BaseRequest<LogonRequest, LogonResponse> implements LogonRequest {
         private Boolean async;
         public LogonRequest async(Boolean async) {
             this.async = async;
@@ -445,6 +494,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -460,7 +515,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new LogonRequestImpl();
     }
     
-    private class MaintenanceRequestImpl implements MaintenanceRequest {
+    private class MaintenanceRequestImpl extends BaseRequest<MaintenanceRequest, MaintenanceResponse> implements MaintenanceRequest {
         private Boolean async;
         public MaintenanceRequest async(Boolean async) {
             this.async = async;
@@ -490,6 +545,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -505,7 +566,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new MaintenanceRequestImpl();
     }
     
-    private class MigrateRequestImpl implements MigrateRequest {
+    private class MigrateRequestImpl extends BaseRequest<MigrateRequest, MigrateResponse> implements MigrateRequest {
         private Boolean async;
         public MigrateRequest async(Boolean async) {
             this.async = async;
@@ -559,6 +620,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -574,7 +641,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new MigrateRequestImpl();
     }
     
-    private class PreviewSnapshotRequestImpl implements PreviewSnapshotRequest {
+    private class PreviewSnapshotRequestImpl extends BaseRequest<PreviewSnapshotRequest, PreviewSnapshotResponse> implements PreviewSnapshotRequest {
         private Boolean async;
         public PreviewSnapshotRequest async(Boolean async) {
             this.async = async;
@@ -648,6 +715,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -663,7 +736,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new PreviewSnapshotRequestImpl();
     }
     
-    private class RebootRequestImpl implements RebootRequest {
+    private class RebootRequestImpl extends BaseRequest<RebootRequest, RebootResponse> implements RebootRequest {
         private Boolean async;
         public RebootRequest async(Boolean async) {
             this.async = async;
@@ -686,6 +759,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -701,7 +780,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new RebootRequestImpl();
     }
     
-    private class RemoveRequestImpl implements RemoveRequest {
+    private class RemoveRequestImpl extends BaseRequest<RemoveRequest, RemoveResponse> implements RemoveRequest {
         private Boolean async;
         public RemoveRequest async(Boolean async) {
             this.async = async;
@@ -733,12 +812,23 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 if (force != null) {
                     uriBuilder.addParameter("force", XmlWriter.renderBoolean(force));
                 }
+                if (query != null) {
+                    for (Map.Entry<String, String> queryParam : query.entrySet()) {
+                        uriBuilder.addParameter(queryParam.getKey(), queryParam.getValue());
+                    }
+                }
                 request = new HttpDelete(uriBuilder.build());
             }
             catch (URISyntaxException ex) {
                 throw new Error("Failed to build URL", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             if (
               response.getStatusLine().getStatusCode() == 200
@@ -761,7 +851,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new RemoveRequestImpl();
     }
     
-    private class ReorderMacAddressesRequestImpl implements ReorderMacAddressesRequest {
+    private class ReorderMacAddressesRequestImpl extends BaseRequest<ReorderMacAddressesRequest, ReorderMacAddressesResponse> implements ReorderMacAddressesRequest {
         private Boolean async;
         public ReorderMacAddressesRequest async(Boolean async) {
             this.async = async;
@@ -784,6 +874,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -799,7 +895,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new ReorderMacAddressesRequestImpl();
     }
     
-    private class ShutdownRequestImpl implements ShutdownRequest {
+    private class ShutdownRequestImpl extends BaseRequest<ShutdownRequest, ShutdownResponse> implements ShutdownRequest {
         private Boolean async;
         public ShutdownRequest async(Boolean async) {
             this.async = async;
@@ -822,6 +918,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -837,7 +939,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new ShutdownRequestImpl();
     }
     
-    private class StartRequestImpl implements StartRequest {
+    private class StartRequestImpl extends BaseRequest<StartRequest, StartResponse> implements StartRequest {
         private Boolean async;
         public StartRequest async(Boolean async) {
             this.async = async;
@@ -900,6 +1002,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -915,7 +1023,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new StartRequestImpl();
     }
     
-    private class StopRequestImpl implements StopRequest {
+    private class StopRequestImpl extends BaseRequest<StopRequest, StopResponse> implements StopRequest {
         private Boolean async;
         public StopRequest async(Boolean async) {
             this.async = async;
@@ -938,6 +1046,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -953,7 +1067,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new StopRequestImpl();
     }
     
-    private class SuspendRequestImpl implements SuspendRequest {
+    private class SuspendRequestImpl extends BaseRequest<SuspendRequest, SuspendResponse> implements SuspendRequest {
         private Boolean async;
         public SuspendRequest async(Boolean async) {
             this.async = async;
@@ -976,6 +1090,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -991,7 +1111,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new SuspendRequestImpl();
     }
     
-    private class ThawFilesystemsRequestImpl implements ThawFilesystemsRequest {
+    private class ThawFilesystemsRequestImpl extends BaseRequest<ThawFilesystemsRequest, ThawFilesystemsResponse> implements ThawFilesystemsRequest {
         private Boolean async;
         public ThawFilesystemsRequest async(Boolean async) {
             this.async = async;
@@ -1014,6 +1134,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -1029,7 +1155,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new ThawFilesystemsRequestImpl();
     }
     
-    private class TicketRequestImpl implements TicketRequest {
+    private class TicketRequestImpl extends BaseRequest<TicketRequest, TicketResponse> implements TicketRequest {
         private Boolean async;
         public TicketRequest async(Boolean async) {
             this.async = async;
@@ -1064,6 +1190,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -1089,7 +1221,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new TicketRequestImpl();
     }
     
-    private class UndoSnapshotRequestImpl implements UndoSnapshotRequest {
+    private class UndoSnapshotRequestImpl extends BaseRequest<UndoSnapshotRequest, UndoSnapshotResponse> implements UndoSnapshotRequest {
         private Boolean async;
         public UndoSnapshotRequest async(Boolean async) {
             this.async = async;
@@ -1112,6 +1244,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to write request", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             Action action = checkAction(response);
             EntityUtils.consumeQuietly(response.getEntity());
@@ -1127,7 +1265,7 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
         return new UndoSnapshotRequestImpl();
     }
     
-    private class UpdateRequestImpl implements UpdateRequest {
+    private class UpdateRequestImpl extends BaseRequest<UpdateRequest, UpdateResponse> implements UpdateRequest {
         private Boolean async;
         public UpdateRequest async(Boolean async) {
             this.async = async;
@@ -1161,6 +1299,11 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 if (nextRun != null) {
                     uriBuilder.addParameter("next_run", XmlWriter.renderBoolean(nextRun));
                 }
+                if (query != null) {
+                    for (Map.Entry<String, String> queryParam : query.entrySet()) {
+                        uriBuilder.addParameter(queryParam.getKey(), queryParam.getValue());
+                    }
+                }
                 request = new HttpPut(uriBuilder.build());
             }
             catch (URISyntaxException ex) {
@@ -1178,6 +1321,12 @@ public class VmServiceImpl extends ServiceImpl implements VmService {
                 throw new Error("Failed to parse response", ex);
             }
             
+            if (headers != null) {
+                for (Map.Entry<String, String> header : headers.entrySet()) {
+                    request.setHeader(header.getKey(), header.getValue());
+                }
+            }
+            
             HttpResponse response = getConnection().send(request);
             if (
               response.getStatusLine().getStatusCode() == 200
